//////////////////////////////////////////////////////////////////////////////////////////
//	CreateParticleTexture.cpp
//	return the ID of a particle texture, creating it if necessary. -1 if error
//	Downloaded from: www.paulsprojects.net
//	Created:	16th December 2002
//
//	Copyright (c) 2006, Paul Baker
//	Distributed under the New BSD Licence. (See accompanying file License.txt or copy at
//	http://www.paulsprojects.net/NewBSDLicense.txt)
//////////////////////////////////////////////////////////////////////////////////////////	
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <vector>
#include "../../GL files/glee.h"			//header for OpenGL 1.4
#include <GL/glu.h>
#include "../../Image/IMAGE.h"
#include "../../Maths/Maths.h"
#include "../../List/LIST.h"
#include "../RENDER_MANAGER.h"

int RENDER_MANAGER::CreateParticleTexture(char * filename)
{
	//See if this texture has already been created
	//If so, return its ID
	for(std::size_t i=0; i<particleData.textures.size(); ++i)
	{
		if(strcmp(filename, particleData.textures[i].filename)==0)
			return i;
	}

	//If not, create temporary 2d texture
	TEXTURE_2D tempParticleTexture;

	//Save the filename into the temporary texture
	strcpy(tempParticleTexture.filename, filename);

	//Load our temporary texture
	if(!tempParticleTexture.Load())
		return -1;

	//If loaded correctly, save the temp texture into the vector
	particleData.textures.push_back(tempParticleTexture);

	//Also create a new index list for this texture
	LIST <unsigned int> tempIndexList;
	particleData.indexLists.push_back(tempIndexList);

	//return the ID
	return particleData.textures.size()-1;
}
