//////////////////////////////////////////////////////////////////////////////////////////
//	EnableParticleVertexArrays.cpp
//	Enable particle vertex arrays
//	Downloaded from: www.paulsprojects.net
//	Created:	23rd December 2002
//
//	Copyright (c) 2006, Paul Baker
//	Distributed under the New BSD Licence. (See accompanying file License.txt or copy at
//	http://www.paulsprojects.net/NewBSDLicense.txt)
//////////////////////////////////////////////////////////////////////////////////////////	
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <vector>
#include "../../GL files/glee.h"			//header for OpenGL 1.4
#include "../../Maths/Maths.h"
#include "../../List/LIST.h"
#include "../RENDER_MANAGER.h"

void RENDER_MANAGER::EnableParticleVertexArrays(bool useColorArray,
												bool useAttenuatedColorArray,
												GLenum textureCoordUnit)
{
	//Disable the current arrays
	DisableParticleVertexArrays();

	//Enable new arrays

	//vertex array
	glVertexPointer(3, GL_FLOAT, sizeof(PARTICLE_VERTEX), &particleData.vertices[0].position);
	glEnableClientState(GL_VERTEX_ARRAY);
	particleVertexArrayEnabled=true;


	//color array
	if(useColorArray)
	{
		glColorPointer(4, GL_FLOAT, sizeof(PARTICLE_VERTEX), &particleData.vertices[0].color);
		glEnableClientState(GL_COLOR_ARRAY);
	}
	
	if(useAttenuatedColorArray)
	{
		glColorPointer(	4, GL_FLOAT, sizeof(PARTICLE_VERTEX),
						&particleData.vertices[0].attenuatedColor);
		glEnableClientState(GL_COLOR_ARRAY);
	}

	//Update boolean record
	particleColorArrayEnabled=useColorArray || useAttenuatedColorArray;

	
	//texture coord array
	if(textureCoordUnit != 0)
	{
		glClientActiveTexture(textureCoordUnit);
		glTexCoordPointer(2, GL_FLOAT, sizeof(PARTICLE_VERTEX), &particleData.vertices[0].texCoords);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	}

	//Update current array
	currentParticleTextureCoordUnit=textureCoordUnit;

	//restore client active texture
	glClientActiveTexture(GL_TEXTURE0);
}